# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
Exceptions generated by the invirtualenv module
"""

class InvirtualenvError(Exception):
    """
    An invirtualenv error occurred.
    """
    pass

class AlreadyExists(InvirtualenvError):
    """
    Package already Exists exception
    """
    pass


class BuildException(InvirtualenvError):
    """
    There was an error Building the VirtualEnv
    """
    pass


class CommandNotFound(InvirtualenvError):
    """
    The command specified wasn't found
    """
    pass


class InsufficientPermissions(InvirtualenvError):
    """
    The current user has insufficient permissions to complete the specified
    operation.
    """
    pass


class NoPackageVersions(InvirtualenvError):
    """
    Query of the Python PyPi repo returned no packages with the specified
    name.
    """
    pass


class PackageConfigFailure(InvirtualenvError):
    """
    Unable to create a package configuration file
    """
    pass


class PackageGenerationFailure(InvirtualenvError):
    """
    Package generation failed
    """
    pass
